/*****************************************************************************
*
*	File Name:   main.c                                        
*                                                                  
*	Content:     Test code to verify operation of MDECE30262 board, MSV30262-SKP board.                
*                                                                  
*	Copyright (c) 2002 Mitsubishi Electric  Electronics USA, Inc.                  
*	All rights reserved
*
*	Mitsubishi Electric  Electronics USA does not guarantee the performance or 
*	use of this source-code.  The intended use of provided source-code is the 
*	sole responsible of the user.  The files have been successfully compiled 
*	using Mitsubishi's NC30 compiler.  Before using this software review the 
*   	source and make any necessary changes to support your hardware and application.
*                                                               
*=============================================================================
*	$Log: main.c,v $
*	Revision 1.1  2002-06-06 16:43:28-04  bembry
*   Revision 1.2  2002-09-23 		      rseville - Modified for MSV30262-SKP
*   Revision 1.3  2002-10-18 		      rseville - for common files directory
*	Initial revision
*
*===========================================================================*/
#include "skp26.h"
#include "serial26.h"

void main(void);
void init_ad_reads(void);
void display_ad_value(void);
#pragma INTERRUPT ADC_intr
void	ADC_intr(void);

void led_blinker(void);

enum MODS{CK_SWLEDs = 0, READ_POT_ADJUST, READ_TEMP_RT1, RS232_TEST, CLOCK_TEST, XC_OSC, RC_OSC}; 
char mode, DELAYED, init_device, rec_char, num_char,led_blink,clock_fault_reset;
long i, program_delay;
const char message[15][9] = {
{"  MEUS  "},{"(c) 2002"},	  // 0, 1
{"1: SW & "},{"LED TST "},	  // 2, 3 
{"2: POT  "},{"3: TEMP "},	  // 4, 5
{"4: RS232"},{"5: M-OSC"},	  // 6, 7
{"RM RES  "},{"        "},	  // 8, 9
{"STOPPED "},{"RUNNING "},	  //10,11
{"6:XC OSC"},{"7:RC OSC"},    //12,13
{"INS-J3&4"}};				  //14
const char num_to_char[11] = {'0','1','2','3','4','5','6','7','8','9',' '};
const unsigned int base_ten[5] = {10000,1000,100,10,1};

unsigned int measured_value;

void main(void)
{
pd8_6 = 1;					// set XCin/out ports to outputs
pd8_7 = 1;
pd6_4 = 1;					// set RTS1 to output - for USB Monitor's Running LED

prc0 = 1;					// Enable writes to Clock control registers.
cm04 = 1;					// Set IO port to XCin and XCout - enables 32 Khz clock. 
prc0 = 0;					// Inhibits writes to Clock control registers. 

ENABLE_LEDS
ALL_LED_ON 
mode =  CK_SWLEDs;			// Intialize test mode.
init_disp();
display(0, message[0]);
display(1, message[1]);
i=0;

if(cm22 == 1)  					// ring OSC 
	while(i < 50000) { i++; asm("nop"); }
else
	while(i < 1000000) { i++; asm("nop"); }

startover:
mode = 0; i=0;
ENABLE_LEDS
ALL_LED_OFF
display(0, message[2]);
display(1, message[3]);
if(cm21 == 1)  					// ring OSC  
	program_delay = 250;
else
	program_delay = 25000;

while(1) {						// main program loop

	++i;
	DELAYED = 0;
	if(i >= program_delay) {		// delay.  
		i = 0;
	 	DELAYED = 1;
	}

	switch(mode){
		case CK_SWLEDs:
			if(S_S2==0)
				LED(RED_LED, ON);
			else
				if((RED_LED == 0) && (DELAYED == 1))
					LED(RED_LED, OFF);
			if(S_S3==0)
				LED(YEL_LED, ON);
			else 	
				if((YEL_LED == 0)&&(DELAYED == 1))
					LED(YEL_LED, OFF); 	
			if(S_S4==0)
				LED(GRN_LED, ON); 	
			else
				if((GRN_LED == 0)&&(DELAYED == 1))
					LED(GRN_LED, OFF);
			if((S_S2==0)&&(S_S3==0)){
				while(S_S2==0);
				mode = READ_POT_ADJUST;
				DISABLE_LEDS
				disp_ctrlw(LCD_CLEAR);
				disp_ctrlw(LCD_CURSOR_RIGHT);
				display(0, message[4]);
				init_device = 1;
			} 	
			break;

		case READ_POT_ADJUST:			// ADC Demo: AN1 - Analog Adjust Potentiometer - R46
			if(init_device){
				init_device = 0;
				if(cm22 == 1)  			// ring OSC 
					program_delay = 500;
				else
					program_delay = 50000;
				adcon0 = 0x01;        	// AN1, Single Shot
				init_ad_reads();	  	// finish AD initialization 	
			}
			if(DELAYED)					// display AD value and start next conversion.  
				display_ad_value();

			if(S_S2==0){
				while(S_S2==0);
				mode = READ_TEMP_RT1;
				DISABLE_LEDS
				disp_ctrlw(LCD_CLEAR);
				disp_ctrlw(LCD_CURSOR_RIGHT);
				display(0, message[5]);
				init_device = 1;
			}
			break;

		case READ_TEMP_RT1:				// ADC Demo: AN0 - Thermistor RT1
			if(init_device){
				init_device = 0;
				if(cm22 == 1)  			// ring OSC 
					program_delay = 500;
				else
					program_delay = 50000;
				adcon0 = 0x00;        	// AN0, Single Shot
				init_ad_reads();	  	// finishes initialization of AD reads. 	
			}
			if(DELAYED)					// display AD value and start next conversion.  
				display_ad_value();
			if(S_S2==0){
				while(S_S2==0);
				mode = RS232_TEST;
				DISABLE_LEDS
				disp_ctrlw(LCD_CLEAR);
				disp_ctrlw(LCD_CURSOR_LEFT);
				display(0, message[6]);
				init_device = 1;
			}
			break;

		case RS232_TEST:
			if(init_device){
				init_device = 0;
				init_uart();
				display(1, message[9]);
				disp_ctrlw(LCD_HOME_L2);
				num_char = 0;
			}
			if(char_recv()){	
				rec_char = 	recv_schar();
				xmit_schar(rec_char);
				if(++num_char >= 9){
					display(1, message[9]);
					disp_ctrlw(LCD_HOME_L2);
					num_char = 1;
				}
				disp_dataw(rec_char);
			}
			if(S_S2==0){
				while(S_S2==0);
				u0c1 &= 0x00;    			// receive disable
				s0ric = 0x00;				// disable receive interrupt
				mode = CLOCK_TEST;
				DISABLE_LEDS
				disp_ctrlw(LCD_CLEAR);
				disp_ctrlw(LCD_CURSOR_LEFT);
				display(0, message[7]);
				init_device = 1;
			}
 			break;

		case CLOCK_TEST:
			if(init_device){
			    program_delay = 20000;
				clock_fault_reset = 0;
				led_blink = 0;
				init_device = 0;
				prc0 = 1;						// Enable writes to Clock control registers.
				cm2 = 0x03;						// Oscillation stop detection enabled.
				prcr = 0x00;					// Inhibit writes to Clock control registers.
			}
			if(DELAYED){
				if(cm23){
					display(1, message[10]);	// OSC stopped.
					program_delay = 7000;
				} 
				else {
					if(clock_fault_reset == 1){
  						prc0 = 1;				// Enable writes to Clock control registers.
 						cm2 = 0x00;				// Select Xin clock 
 						prcr = 0x00;			// Inhibit writes to Clock control registers.
						init_device = 1;
					}
					else {
						if(cm22 == 1) 			// osc stop detected
							clock_fault_reset = 1;
					}
					display(1, message[11]);
 				}
			}

			led_blinker();
			if(S_S2==0){
				while(S_S2==0);
				program_delay = 25000;
				init_device = 1;
				prc0 = 1;					// Enable writes to Clock control registers.
				cm2 = 0x00;					// Oscillation stop detection inhibited. 
				prcr = 0x00;				// Inhibit writes to Clock control registers.
				goto startover;
			}

			if(S_S3==0){
				while(S_S3==0);
				program_delay = 25000;
				init_device = 1;
				mode = XC_OSC;
				display(0, message[12]);
			}

			if(S_S4==0){
				while(S_S4==0);
				program_delay = 25000;
				init_device = 1;
				mode = RC_OSC;
				display(0, message[13]);
			}
			break;

		case XC_OSC:						// use XCin instead of Xin, S5 must be in XC
			if(init_device){
			    program_delay = 10;
				clock_fault_reset = 0;
				led_blink = 0;
				init_device = 0;
				prc0 = 1;					// Enable writes to Clock control registers.
				cm07 = 1;					// Switch from Xin to XCin
				cm2 = 0x00;					// Disable osc stop detection, ring osc
				prcr = 0x00;				// Inhibit writes to Clock control registers.
			}
			if(DELAYED) 
				led_blinker();
			if(S_S4==0){
				while(S_S4==0);
				program_delay = 25000;
				mode = CK_SWLEDs;
				prc0 = 1;					// Enable writes to Clock control registers.
				cm07 = 0;					// Switch from XCin to Xin
   				cm2 = 0x03;					// Ring osc, stop detection enabled.
				prcr = 0x00;				// Inhibit writes to Clock control registers.
				display(0, message[2]);
 				display(1, message[3]);
			}
			break;

		case RC_OSC:
			if(init_device){
			    program_delay = 20000;
				clock_fault_reset = 0;
				led_blink = 0;
				init_device = 0;
				prc0 = 1;					// Enable writes to Clock control registers.
				cm13 = 0x1;					// Xin equal to RC Oscilator.  
				cm2 = 0x03;					// Ring osc, osc stop detection enabled.
				prcr = 0x00;				// Inhibit writes to Clock control registers.
			}
			if(DELAYED){
				if(cm22){
					display(1, message[10]);	// RC OSC stopped.
					program_delay = 3000;
				} 
				else {
					if(clock_fault_reset == 1){
  						prc0 = 1;					// Enable writes to Clock control registers.
 						cm2 = 0x00;					// Select Xin clock 
 						prcr = 0x00;				// Inhibit writes to Clock control registers.
						init_device = 1;
					}
					else {
						if(cm22 == 1) 
							clock_fault_reset = 1;
					}
					display(1, message[11]);
 				}
 				led_blinker();
 			}
			if(S_S4==0){
				while(S_S4==0);
				program_delay = 25000;
				mode = CK_SWLEDs;
				prc0 = 1;					// Enable writes to Clock control registers.
				cm07 = 0;					// Switch from XCin to Xin
   				cm2 = 0x00;					// Ring osc, stop detection enabled.
				prcr = 0x00;				// Inhibit writes to Clock control registers.
				display(0, message[2]);
 				display(1, message[3]);
			}
			break;
		}
	}
}

/*****************************************************************************
Name:         init_ad_reads  
Parameters:                     
Returns:        
Description:  Initalizes the ad port.  Does not set ADCON0.  
*****************************************************************************/
void init_ad_reads(void)
{
	adcon1 = 0x28;        	// Vref connected, fad/s selected, 10bit mode
	adcon2 = 0x01;		  	// With sample and hold. 
	display(1, message[9]);	// Clear line two of display. 
	asm("FCLR I");			// disable interrupts
	adic = 0x02;			// Enable interrupt control ADC complete.
	asm("FSET I");			// enable interrrupts
	adst = 1;			  	// start first ad conversion. 
}

/*****************************************************************************
Name:			display_ad_value           
Parameters:                     
Returns:        
Description:    Display AD value on line two of LCD. 
*****************************************************************************/
void display_ad_value(void)
{
	unsigned int result;
	char j;
 	disp_ctrlw(LCD_HOME_L2);
 	disp_dataw(num_to_char[10]);
 	disp_dataw(num_to_char[10]);
 	for(j=1;j<5;j++){		   // Calculate 4 digit result. 
 		result =  measured_value / base_ten[j];
 		disp_dataw(num_to_char[result]);
 		measured_value  = measured_value - 	(result * base_ten[j]);
 	}

	adst = 1;			  // start next ad conversion.
}

/*****************************************************************************
Name:          ADC_intr 
Parameters:                     
Returns:        
Description:   Interrupt handler for ADC test. Subtracting read AD value from
			   0x3FF since our reference is Vcc.  
*****************************************************************************/
void	ADC_intr(void){

	if (ch0 == 0)						// using AN0?
		measured_value = (0x3FF - ad0);	// read AD0
	
	else
		measured_value = (0x3FF - ad1);	// read AD1 from 0x3FF to fix pot turns CW:Max/CCW:Min
}

/*****************************************************************************
Name:        LED_blink   
Parameters:                     
Returns:        
Description: Blinks RED YELLOW and GREEN LEDs   
*****************************************************************************/
void led_blinker(void)
{
	switch(led_blink){
		case 0:
			ALL_LED_OFF
			led_blink = 1;
			break;
		case 1:
 			led_blink = 2; 
			LED(RED_LED, ON);
 			break;
 		case 2:
			LED(YEL_LED, ON);
			led_blink = 3;
			break;
		case 3:
			default:
			LED(GRN_LED, ON);
			led_blink = 0;
			break;
			}
}